    <!DOCTYPE html>
    <html lang="en">
        <head>
            <title>About</title>
            <style type="text/css">a:link {color: #6699CC;}</style>
        </head>    
    <body style="font-weight:400; font-style:normal;">
        <p style="font-size:22pt"><strong>{{ app_name }} {{version}}</strong></p>
        <p>Copyright (C), 2023, {{author}}.</p>
        <p>Licensed under the <b>LGPL V3</b>.</p>
        <p>Powered by <b>Qt6</b> via <b>PySide6</b> bindings under LGPL V3 license.</p>
        <p>Assets:</p>
        <ul>
            <li><a href="https://fonts.google.com/">Noto Sans and Noto Sans Mono fonts</a> by Google under <a href="https://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&id=OFL">SIL Open Font License</a></li>
            <li><a href="https://fonts.google.com/">Material icons</a> by Google under <a href="https://www.apache.org/licenses/LICENSE-2.0">Apache License 2.0</a></li>
            <li><a href="https://www.svgrepo.com/svg/340563/linux-alt/">Linux icon</a> by Carbon Design under <a href="https://github.com/carbon-design-system/carbon/blob/main/LICENSE">Apache License 2.0</a></li>
            <li><a href="https://www.svgrepo.com/svg/488495/apple">Apple icon</a> by Klever Space under <a href="https://opensource.org/licenses/MIT">MIT License</a></li>
            <li><a href="https://www.svgrepo.com/svg/488736/windows">Windows icon</a> by Klever Space under <a href="https://opensource.org/licenses/MIT">MIT License</a></li>
            <li><a href="https://www.svgrepo.com/svg/487645/package">Package icon</a> by Neuicons under <a href="https://github.com/neuicons/neu/blob/master/LICENSE">MIT License</a></li>
            <li><a href="https://www.svgrepo.com/svg/383786/open-box-parcel">Open Box icon</a> by wishforge.games under <a href="https://creativecommons.org/licenses/by/4.0/legalcode">CC Attribution License</a></li>
        </ul>
        <p >For more information visit <a href="{{ repo_url }}">{{ repo_url }}</a>.</p>
    </body>
    </html>